#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "trie_sedgewick.h"

// Our alphabet size.  One byte's worth.
#define RADIX 256

struct trie_node
   {
   void * data;
   struct trie_node * next[RADIX];
   };

struct trie
   {
   struct trie_node * root;
   };


struct trie_node * trie_node_new();
void * trie_node_get(struct trie_node * node, char * key, int d);
struct trie_node * trie_node_put(struct trie_node * node, char * key, void * data, int d);

////////////////////////////////////////////////////////////////////////////////
struct trie_node *
trie_node_new()
   {
   struct trie_node * node = malloc(sizeof(struct trie_node));
   if (NULL == node)
      return NULL;

   for (int i = 0; i < RADIX; ++i)
      node->next[i] = NULL;
   node->data = NULL;

   return node;
   }

////////////////////////////////////////////////////////////////////////////////
struct trie *
trie_new()
   {
   struct trie * trie = malloc(sizeof(struct trie));
   if (NULL == trie)
      return NULL;

   trie->root = NULL;
   return trie;
   }

////////////////////////////////////////////////////////////////////////////////
void
trie_delete(struct trie * trie)
   {
   if (NULL == trie)
      return;
   // TODO free nodes
   free(trie);
   }

////////////////////////////////////////////////////////////////////////////////
char *
trie_put(struct trie * trie, char * key, void * data)
   {
   if ( (NULL == trie) || (NULL == key) )
      return NULL;
   trie->root = trie_node_put(trie->root, key, data, 0);
   return key;
   }

////////////////////////////////////////////////////////////////////////////////
struct trie_node *
trie_node_put(struct trie_node * node, char * key, void * data, int d)
   {
   if (NULL == node)
      node = trie_node_new();
   if (NULL == node)
      return NULL;

   if (d == strlen(key))
      {
      node->data = data;
      return node;
      }
   int c = key[d];
   node->next[c] = trie_node_put(node->next[c], key, data, d+1);
   return node;
   }

////////////////////////////////////////////////////////////////////////////////
void *
trie_get(struct trie * trie, char * key)
   {
   if ( (NULL == trie) || (NULL == key) )
      return NULL;

   struct trie_node * node = trie_node_get(trie->root, key, 0);
   if (NULL == node)
      return NULL;
   return node->data;
   }

////////////////////////////////////////////////////////////////////////////////
void *
trie_node_get(struct trie_node * node, char * key, int d)
   {
   if (NULL == node)
      return NULL;
   if (d == strlen(key))
      return node;
   return trie_node_get(node->next[(int) key[d]], key, d+1);
   }
